#!/usr/bin/perl -w
#
# fixsrc.pl - fixes epoc src tree to lower case
#
# (C) 2001 Mal Minhas
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
# 0.01   16. Nov 2001   alfredh   original version
#
use strict;
use warnings;

unless ($#ARGV == 1){
    die "Usage: fixsrc <srcdir> <targetdir>\n";
}
unless (-d $ARGV[0]){
    die "Supplied <srcdir> is not a directory\n";
}
my $targetdir = $ARGV[1];
# make the target directory with "chmod a=rwx"
mkdir $targetdir,0777; 
# start process of moving directories over
my $srcdir = $ARGV[0];
my $retval=movedir($srcdir,$targetdir);
print "Returned OK\n";


#
# recursive open dir and merge of directory contents
#
sub movedir {
    my $srcdir = shift;
    my $targetdir = shift;
    my @srcdirs;

    unless (opendir(SRC,$srcdir)){
	warn "Couldn't open directory $srcdir:$!\n";
    }
    @srcdirs = readdir SRC;
    closedir (SRC);

    foreach my $i (@srcdirs){
	if (($i eq ".") || ($i eq "..")) { # skip '.' and '..'
	    next;
	}
	my $j = $i;

	$j =~ tr/[A-Z]/[a-z]/;  # force to lower case
	$i = "$srcdir/$i";
	$j = "$targetdir/$j";
	print "src: $i, tgt: $j\n";
	
	if (-f $i) { # file
	    if (-e $j){
		print "$j already exists - ignoring $i\n";
	    }
	    else {
		system("cp",$i,$j);
		# now we need to downinch the target file (only text files)
		if (-T $j){
		  downinch($j);
		}
	    }
	} 
	elsif ( -d $i) { # directory
	    # if $j doesn't exist at target, need to make it...
	    if (! -e $j){
		print "Creating new directory $j...\n";
		mkdir $j,0777;
	    }
	    movedir($i,$j);
	} 
	else {
	    print STDERR "problems: $i\n";
	}
    }
}


#
# First check that we have a suitable candidate for downinching...
#
sub downinch {
    my $file = shift;
    my $filename = $file;

    if (!-T $file){
      die "internal error in fixsrc.pl - should not downinch binary files";
    }

    # convert from DOS to Unix format
    unixifyfile($file);

    #
    # handle .mmp files
    #
    if ($file =~ /\.(mmp|mmpi|mph)$/){
	my $nfile="$file.tmp";
	# open file and go through line at a time
	open(FILE, "<$file") or die "Error on opening $file $!\n";
	my $nline;
	open(NFILE,">$nfile") or die "Error on opening $nfile $!\n"; 
	while (my $line = <FILE>){
	    if ($line =~ /^(SOURCEPATH|SOURCE|TARGETTYPE|TARGETPATH|TARGET|USERINCLUDE|SYSTEMINCLUDE|LIBRARY|DEFFILE|RESOURCE|START.*BITMAP|SYSTEMRESOURCE|LANG|UID|EPOCSTACKSIZE|DOCUMENT|START.*WINS|END|BASEADDRESS|AIF)(.*)/i){
		# gets the lines with keywords in .mmp
		my $upper = uc($1); # switch up bogus lower case on keywords
		my $lower = lc($2);
		$nline = "$upper$lower\n";
		#print "Line was: \"$line\".  Now is: \"$nline\"\n";
	    }
	    elsif ($line =~ /(.*)\.mmp/i){
                # gets the first comment of .mmp
		$nline = lc($line);
	    }
	    else {
		# leave everything else alone
		$nline = $line;
	    }
#	    $nline =~ s/\\/\//g; # global replace of \ with / in line.
	    print NFILE $nline;
	}
	close FILE;
	close NFILE;
	unlink $file;
	system("mv",$nfile,$filename);
	return;
    }
    #
    # handle bld.inf files
    #
    if ($file =~ /bld\.inf$/){
	# do whatever needs to be done for bld.inf files
	my $nfile="$file.tmp";
	# open file and go through line at a time
	open(INFFILE, "<$file") or die "Error on opening $file $!\n";
	my $nline;
	open(NINFFILE,">$nfile") or die "Error on opening $nfile $!\n"; 
	while (my $line = <INFFILE>){
	    if ($line =~ /^(PRJ_|MAKEFILE)/i){
		# leave the keywords alone
		$nline = $line;
	    }
	    elsif ($line =~ /^(\/\/)/){
		# leave the comments alone
		$nline = $line;
		$nline =~ s/BLD\.INF/bld\.inf/g;
		}
	    else {
		$nline = lc($line);
	    }
#	    $nline =~ s/\\/\//g; # global replace of \ with / in line.
	    print NINFFILE $nline;
	}
	close INFFILE;
	close NINFFILE;
	unlink $file;
	system("mv",$nfile,$filename);
	return;
    }

    # otherwise we (probably!) have a .cpp or .inl or .h file and need to downinch it.
    downinchfile($file)
}


#
# lower case the following TEXT: #include <TEXT> or #include "TEXT"
#
sub downinchfile() {
    my $file = shift;
    my $filename = $file;

    my $nfile="$file.tmp";
    print "Downinching $file...\n";
    # open file and go through line at a time
    open(INFFILE, "<$file") or die "Error on opening $file $!\n";
    my $nline;
    open(NINFFILE,">$nfile") or die "Error on opening $nfile $!\n"; 
    while (my $line = <INFFILE>){
	if ($line =~ /^(\#include\s*)/){
	    $nline = lc($line);
	    $nline =~ s/\\/\//g; # global replace of \ with / in line.
	    #print "Changing \"$line\" to \"$nline\"\n";
	}
	else {
	    $nline = $line;
	}
	print NINFFILE $nline;
    }
    close INFFILE;
    close NINFFILE;
    unlink $file;
    system("mv",$nfile,$filename);
}


#
# Global search and replace of \r\n with \n in text files
#
sub unixifyfile() {
    my $file = shift;
    my $unixify;

    # no \n delimiters
    local $/=undef;
    #undef $/; # alias $INPUT_RECORD_SEPARATOR, $RS

    # open and slurp the file into $_
    open(FILE, $file) or die "Error: $!\n";
    $_ = <FILE>;
    close FILE;

    # do the regexp here.  suffix: globally and multiline
    $unixify = s/\r\n/\n/g;
    if ($unixify){
	system("chmod", "a+w", $file);
	print "unixified $file\n";
	open FOUT, ">$file";
	print FOUT;
	close FOUT;
    }
}



